@@ -3,6 +3,7 @@ from lost_ds.geometry.api import LOSTGeometries
 from lost_ds.functional.api import(remove_empty,
                                    split_by_empty,
                                    split_by_img_path,
+                                   split_train_test,
                                    split_multilabels,
                                    remap_labels,
                                    remap_img_path,
@@ -46,11 +47,17 @@ from lost_ds.vis.api import (vis_sample,
 from lost_ds.segmentation.api import (semantic_segmentation, 
                                       segmentation_to_lost)
 
-from lost_ds.detection.api import (detection_dataset, bbox_nms)
+from lost_ds.detection.api import (detection_dataset, bbox_nms, coco_eval, 
+                                   voc_eval, voc_score_iou_multiplex)
+
+from lost_ds.masking.api import mask_dataset 
 
 from lost_ds.util import (get_fs, 
                           to_parquet)
 
 from lost_ds.core import LOSTDataset
 
-__version__='0.4.0'
+from lost_ds.compat import old_lds_to_new_lds
+
+
+__version__='0.5.0'
