@@ -1,4 +1,5 @@
 from os import name
+from random import random
 import numpy as np
 
 from lost_ds.io.file_man import FileMan
@@ -12,6 +13,7 @@ from lost_ds.functional.validation import (validate_geometries,
 
 from lost_ds.functional.split import (split_by_empty,
                                       split_by_img_path,
+                                      split_train_test,
                                       split_multilabels)
 
 from lost_ds.functional.filter import (remove_empty,
@@ -51,7 +53,9 @@ from lost_ds.segmentation.semantic_seg import (semantic_segmentation,
 from lost_ds.segmentation.anno_from_seg import (segmentation_to_lost,
                                                )
 
-from lost_ds.detection.detection import detection_dataset
+from lost_ds.detection.detection import detection_dataset, bbox_nms
+
+from lost_ds.masking.masking import mask_dataset
 
 from lost_ds.util import get_fs, to_parquet
 
@@ -77,7 +81,7 @@ class LOSTDataset(object):
         try:
             self._parse_data()
         except TypeError as e:
-            pass
+            print('parsing: ', e)
         self.geom = LOSTGeometries()
         self.cropper = DSCropper(self.geom, self.fileman)
         
@@ -86,11 +90,24 @@ class LOSTDataset(object):
     #
     
     def _parse_data(self):
-        # parse anno_data (lists) to numpy array
+        # parse lds-serialized data (2D-lists) to numpy array
+        def _parse(x):
+            try: 
+                return np.vstack(x).squeeze()
+            except:
+                return x
+        
+        parse_keys = [k for k in self.df.keys() if k.endswith('_lds_serialized')]
+        if parse_keys:
+            col_mapper = dict()
+            for k in parse_keys:
+                self.df[k] = self.df[k].apply(lambda x: _parse(x))
+                col_mapper[k] = k.replace('_lds_serialized', '')
+            self.df.rename(col_mapper, axis=1, inplace=True)
+            
         if 'anno_data' in self.df:
-            self.df.anno_data = self.df.anno_data.apply(
-                lambda x: np.vstack(x).squeeze())
-                    
+            self.df.anno_data = self.df.anno_data.apply(lambda x: _parse(x))
+        
             
     def to_parquet(self, path, df=None):
         ''' Store dataset as .parquet file
@@ -194,18 +211,17 @@ class LOSTDataset(object):
     #   Data copy
     #
     
-    def copy_imgs(self, out_dir, df=None, col='img_path', copy_path=None, 
+    def copy_imgs(self, out_dir, df=None, col='img_path', 
                   force_overwrite=False):
         '''Copy all images of dataset into out_dir
 
         Args:
             df (pd.DataFrame): dataframe to copy
             out_dir (str): Destination folder to store images
-            copy_path (str, optional): directory where to copy all images
             col (str): column containing paths to files
         '''
         df = self._get_df(df)
-        copy_imgs(df=df, out_dir=out_dir, col=col, copy_path=copy_path,
+        copy_imgs(df=df, out_dir=out_dir, col=col, 
                   force_overwrite=force_overwrite, filesystem=self.fileman)
             
     
@@ -286,14 +302,14 @@ class LOSTDataset(object):
 
         Args:
             imgs (list): list of imgs to select
-            invers (bool): get the selection if True, get the rest if False 
+            inverse (bool): get the selection if True, get the rest if False 
             df (pd.DataFrame): Frame to apply image selection
 
         Returns:
             pd.DataFrame: dataframe with image selection
         '''
         df = self._get_df(df)
-        df = img_selection(imgs=imgs, df=df, invers=inverse)
+        df = img_selection(imgs=imgs, df=df, inverse=inverse)
         return self._update_inplace(df, inplace)
     
     
@@ -374,7 +390,7 @@ class LOSTDataset(object):
         '''
         df = self._get_df(df)
         df = remap_labels(df, label_mapping, col, dst_col)
-        self._update_inplace(df, inplace)
+        return self._update_inplace(df, inplace)
     
     
     #
@@ -404,7 +420,7 @@ class LOSTDataset(object):
         '''
         df = self._get_df(df)
         df = to_rel(df, path_col, self.fileman, verbose)
-        self._update_inplace(df, inplace)
+        return self._update_inplace(df, inplace)
     
     
     def polygon_to_bbox(self, df=None, dst_style=None, inplace=False):
@@ -486,8 +502,28 @@ class LOSTDataset(object):
         df = self._get_df(df)
         return split_by_empty(df, col)
     
-    
-    def split_by_img_path(self, test_size=0.2, val_size=0.2, df=None):
+    def split_train_test(self, test_size=0.2, val_size=0.2, stratify_col=None, 
+                         df=None, col='img_path', random_state=42):
+        '''Split dataset based on img paths (for dataset with multiple 
+            entries for one image)
+        Args:
+            test_size (float): fraction of images in df that will be used for 
+                test dataset
+            val_size (float): fraction of images in df that will be used for 
+                test dataset
+            stratify_col (str): column to use for stratify split
+            df (pd.DataFrame): Dataframe to split
+        Returns: 
+            tuple: pd.DataFrames with dataframe split (train, test, val).
+            if a size is 0.0 it will return None at the according place
+        '''
+        df = self._get_df(df)
+        return split_train_test(test_size=test_size, val_size=val_size, 
+                                stratify_col=stratify_col, df=df, col=col,
+                                random_state=random_state)
+        
+    def split_by_img_path(self, test_size=0.2, val_size=0.2, df=None, 
+                          random_state=42):
         '''Split dataset based on img paths (for dataset with multiple 
             entries for one image)
         Args:
@@ -501,7 +537,8 @@ class LOSTDataset(object):
             if a size is 0.0 it will return None at the according place
         '''
         df = self._get_df(df)
-        return split_by_img_path(test_size=test_size, val_size=val_size, df=df)
+        return split_by_img_path(test_size=test_size, val_size=val_size, df=df, 
+                                 random_state=random_state)
     
     
     def split_multilabels(self, lbl_mapping, df=None, col='anno_lbl', inplace=False):
@@ -616,8 +653,8 @@ class LOSTDataset(object):
     #
     
     def vis_and_store(self, out_dir, df=None, lbl_col='anno_lbl', 
-                      color=(0, 0, 255), line_thickness=2, fontscale=2, 
-                      radius=2):
+                      color=(0, 0, 255), line_thickness='auto', 
+                      fontscale='auto', radius=2, parallel=-1):
         '''Visualize annotations and store them to a folder
 
         Args:
@@ -632,7 +669,7 @@ class LOSTDataset(object):
         '''
         df = self._get_df(df)
         vis_and_store(df, out_dir, lbl_col, color, line_thickness, fontscale, 
-                      self.fileman, radius)
+                      self.fileman, radius, parallel)
     
     
     def vis_semantic_segmentation(self, out_dir, n_classes, palette='dark', 
@@ -658,7 +695,7 @@ class LOSTDataset(object):
     #
     
     def crop_dataset(self, dst_dir, crop_shape=(500, 500), overlap=(0, 0), 
-                     df=None, write_emtpy=False, fill_value=0, inplace=False):
+                     df=None, write_empty=False, fill_value=0, inplace=False):
         """Crop the entire dataset with fixed crop-shape
 
         Args:
@@ -675,7 +712,7 @@ class LOSTDataset(object):
         """
         df = self._get_df(df)
         df = crop_dataset(df=df, dst_dir=dst_dir, crop_shape=crop_shape, 
-                          overlap=overlap, write_empty=write_emtpy, 
+                          overlap=overlap, write_empty=write_empty, 
                           fill_value=fill_value, filesystem=self.fileman)
         return self._update_inplace(df, inplace)
     
@@ -701,7 +738,8 @@ class LOSTDataset(object):
     def crop_components(self, dst_dir, base_labels=-1, lbl_col='anno_lbl', 
                         context=0, df=None, context_alignment=None, 
                         min_size=None, anno_dtype=['polygon'], 
-                        center_lbl_key='center_lbl', inplace=False):
+                        center_lbl_key='center_lbl', inplace=False, 
+                        parallel=-1):
         """Crop the entire dataset with fixed crop-shape
 
         Args:
@@ -731,7 +769,7 @@ class LOSTDataset(object):
         df = self._get_df(df)
         df = crop_components(df, dst_dir, base_labels, lbl_col, context, 
                              context_alignment, min_size, anno_dtype, 
-                             center_lbl_key, self.fileman)
+                             center_lbl_key, self.fileman, parallel=parallel)
         return self._update_inplace(df, inplace)
         
     #
@@ -739,9 +777,10 @@ class LOSTDataset(object):
     #
     
     def semantic_segmentation(self, order, dst_dir, fill_value, df=None, 
-                              anno_dtypes=['polygon'], lbl_col='anno_lbl', 
-                              dst_path_col='seg_path', dst_lbl_col='seg_lbl', 
-                              line_thickness=None, radius=None, inplace=False):
+                              anno_dtypes=['polygon'], use_empty=False, 
+                              lbl_col='anno_lbl', dst_path_col='seg_path', 
+                              dst_lbl_col='seg_lbl', line_thickness=None, 
+                              radius=None, inplace=False):
         '''Create semantic segmentations from polygon-annos
     
         Args:
@@ -774,9 +813,10 @@ class LOSTDataset(object):
                 segmentation looked up in order for creation
         '''
         df = self._get_df(df)
-        df = semantic_segmentation(order, dst_dir, fill_value, df, anno_dtypes, 
-                                   lbl_col, dst_path_col, dst_lbl_col, 
-                                   line_thickness, radius, self.fileman)
+        df = semantic_segmentation(order, dst_dir, fill_value, df, anno_dtypes,
+                                   use_empty, lbl_col, dst_path_col, 
+                                   dst_lbl_col, line_thickness, radius, 
+                                   self.fileman)
         return self._update_inplace(df, inplace)
     
     
@@ -845,4 +885,49 @@ class LOSTDataset(object):
         df = self._get_df(df)
         df = detection_dataset(df, lbl_col, det_col, bbox_style, 
                                use_empty_images, self.fileman)
+        return self._update_inplace(df, inplace)
+    
+    
+    def bbox_nms(self, lbl_col='anno_lbl', method='nms', iou_thr=0.5, df=None, 
+                 inplace=False, parallel=-1, **kwargs):
+        """apply nms to dataframe to suppress overlapping bboxes
+
+        Args:
+            df (pd.DataFrame): dataframe containing bboxes to supress
+            lbl_col (str, optional): columns containing labels. Defaults to 'anno_lbl'.
+            method (str, optional): one of {'nms', 'nmw', 'soft_nms', 'wbf', 'merge'}. Defaults to 'nms'.
+            iou_thr (float, optional): iou threshold when to supress bboxes. Defaults to 0.5.
+            filesystem (fsspec, optional): filesystem. Defaults to None.
+            parallel (int): use parallelism
+
+        Returns:
+            pd.DataFrame: dataframe with overlapping bboxes pruned
+        """
+        df = self._get_df(df)
+        df = bbox_nms(df, lbl_col, method, iou_thr, self.fileman, parallel, 
+                      **kwargs)
+        return self._update_inplace(df, inplace)
+        
+    
+    #
+    #   Masking
+    #
+    
+    def mask_dataset(self, dst_dir, masking_labels, mask_value=0, inverse=False, 
+                     lbl_col='anno_lbl', df=None, inplace=False):
+        """mask images by annos of given label. Either mask the given annos itself 
+        (inverse=False) or everything but the annos (inverse=True)
+
+        Args:
+            dst_dir (str): destination folder
+            masking_labels (list): list of labels to use for masking
+            mask_value (int, optional): pixel value to use for mask. Defaults to 0.
+            inverse (bool, optional): fill the given annos with `mask_value` if True, and inverse if False. Defaults to False.
+            lbl_col (str, optional): column containing the labels. Defaults to 'anno_lbl'.
+            df (pd.DataFrame): dataframe to mask
+            inplace (bool): overwrite self.df with result df
+        """
+        df = self._get_df(df)
+        df = mask_dataset(df, dst_dir, masking_labels, mask_value, inverse, 
+                          lbl_col, self.fileman)
         return self._update_inplace(df, inplace)
\ No newline at end of file
