@@ -1,7 +1,7 @@
 import pandas as pd
 from sklearn.model_selection import train_test_split
 
-from lost_ds.functional.filter import img_selection
+from lost_ds.functional.filter import img_selection, label_selection
 
 
 def split_by_empty(df, col='anno_data'):
@@ -18,8 +18,38 @@ def split_by_empty(df, col='anno_data'):
     not_empty_df = df[~empty_bool]
     return not_empty_df, empty_df
 
+def split_train_test(test_size=0.2, val_size=0.2, stratify_col=None, df=None,
+                     col='img_path', random_state=42):
+    if not len(df):
+        return df, df, df
+    imgs = list(df[col].unique())
+    n_images = len(imgs)
+    stratify = None
+    ids = list(range(n_images))
+    if stratify_col:
+        stratify = list(df[stratify_col])
+        assert len(imgs) == len(df), 'Images cannot occur multiple time in dataset when using stratify!'
+    splits = []
+    for split in [test_size, val_size]:
+        if split:
+            size = int(split * n_images)
+            set_1, set_2, ids_1, ids_2 = train_test_split(imgs, ids, test_size=size, 
+                                                          shuffle=True, 
+                                                          random_state=random_state,
+                                                          stratify=stratify)
+            split_data = label_selection(list(set_2), df=df, col=col)
+            splits.append(split_data)
+            imgs = set_1
+            ids = ids_1
+            if stratify_col:
+                stratify = list(df.iloc[ids_1][stratify_col])
+        else:
+            splits.append(None)
+    train_data = label_selection(list(imgs), df=df, col=col)
+    splits.insert(0, train_data)
+    return tuple(splits)
 
-def split_by_img_path(test_size=0.2, val_size=0.2, df=None):
+def split_by_img_path(test_size=0.2, val_size=0.2, df=None, random_state=42):
     '''Split dataset based on img paths (for dataset with multiple 
         entries for one image)
     Args:
@@ -32,21 +62,8 @@ def split_by_img_path(test_size=0.2, val_size=0.2, df=None):
         tuple: pd.DataFrames with dataframe split (train, test, val).
         if a size is 0.0 it will return None at the according place
     '''
-    imgs = list(df.img_path.unique())
-    n_images = len(imgs)
-    splits = []
-    for split in [test_size, val_size]:
-        if split:
-            size = int(split * n_images)
-            set_1, set_2 = train_test_split(imgs, test_size=size)
-            split_data = img_selection(list(set_2), df=df)
-            splits.append(split_data)
-            imgs = set_1
-        else:
-            splits.append(None)
-    train_data = img_selection(list(imgs), df=df)
-    splits.insert(0, train_data)
-    return tuple(splits)
+    return split_train_test(test_size, val_size, None, df, 'img_path', 
+                            random_state)
 
 
 def split_multilabels(lbl_mapping, df:pd.DataFrame=None, col='anno_lbl'):
